from .EmbThread import EmbThread


def get_thread_set():
    return [
        EmbThreadSew(0, 0, 0, "Unknown", "0"),
        EmbThreadSew(0, 0, 0, "Black", "1"),
        EmbThreadSew(255, 255, 255, "White", "2"),
        EmbThreadSew(255, 255, 23, "Sunflower", "3"),
        EmbThreadSew(250, 160, 96, "Hazel", "4"),
        EmbThreadSew(92, 118, 73, "Green Dust", "5"),
        EmbThreadSew(64, 192, 48, "Green", "6"),
        EmbThreadSew(101, 194, 200, "Sky", "7"),
        EmbThreadSew(172, 128, 190, "Purple", "8"),
        EmbThreadSew(245, 188, 203, "Pink", "9"),
        EmbThreadSew(255, 0, 0, "Red", "10"),
        EmbThreadSew(192, 128, 0, "Brown", "11"),
        EmbThreadSew(0, 0, 240, "Blue", "12"),
        EmbThreadSew(228, 195, 93, "Gold", "13"),
        EmbThreadSew(165, 42, 42, "Dark Brown", "14"),
        EmbThreadSew(213, 176, 212, "Pale Violet", "15"),
        EmbThreadSew(252, 242, 148, "Pale Yellow", "16"),
        EmbThreadSew(240, 208, 192, "Pale Pink", "17"),
        EmbThreadSew(255, 192, 0, "Peach", "18"),
        EmbThreadSew(201, 164, 128, "Beige", "19"),
        EmbThreadSew(155, 61, 75, "Wine Red", "20"),
        EmbThreadSew(160, 184, 204, "Pale Sky", "21"),
        EmbThreadSew(127, 194, 28, "Yellow Green", "22"),
        EmbThreadSew(185, 185, 185, "Silver Grey", "23"),
        EmbThreadSew(160, 160, 160, "Grey", "24"),
        EmbThreadSew(152, 214, 189, "Pale Aqua", "25"),
        EmbThreadSew(184, 240, 240, "Baby Blue", "26"),
        EmbThreadSew(54, 139, 160, "Powder Blue", "27"),
        EmbThreadSew(79, 131, 171, "Bright Blue", "28"),
        EmbThreadSew(56, 106, 145, "Slate Blue", "29"),
        EmbThreadSew(0, 32, 107, "Nave Blue", "30"),
        EmbThreadSew(229, 197, 202, "Salmon Pink", "31"),
        EmbThreadSew(249, 103, 107, "Coral", "32"),
        EmbThreadSew(227, 49, 31, "Burnt Orange", "33"),
        EmbThreadSew(226, 161, 136, "Cinnamon", "34"),
        EmbThreadSew(181, 148, 116, "Umber", "35"),
        EmbThreadSew(228, 207, 153, "Blonde", "36"),
        EmbThreadSew(225, 203, 0, "Sunflower", "37"),
        EmbThreadSew(225, 173, 212, "Orchid Pink", "38"),
        EmbThreadSew(195, 0, 126, "Peony Purple", "39"),
        EmbThreadSew(128, 0, 75, "Burgundy", "40"),
        EmbThreadSew(160, 96, 176, "Royal Purple", "41"),
        EmbThreadSew(192, 64, 32, "Cardinal Red", "42"),
        EmbThreadSew(202, 224, 192, "Opal Green", "43"),
        EmbThreadSew(137, 152, 86, "Moss Green", "44"),
        EmbThreadSew(0, 170, 0, "Meadow Green", "45"),
        EmbThreadSew(33, 138, 33, "Dark Green", "46"),
        EmbThreadSew(93, 174, 148, "Aquamarine", "47"),
        EmbThreadSew(76, 191, 143, "Emerald Green", "48"),
        EmbThreadSew(0, 119, 114, "Peacock Green", "49"),
        EmbThreadSew(112, 112, 112, "Dark Grey", "50"),
        EmbThreadSew(242, 255, 255, "Ivory White", "51"),
        EmbThreadSew(177, 88, 24, "Hazel", "52"),
        EmbThreadSew(203, 138, 7, "Toast", "53"),
        EmbThreadSew(247, 146, 123, "Salmon", "54"),
        EmbThreadSew(152, 105, 45, "Cocoa Brown", "55"),
        EmbThreadSew(162, 113, 72, "Sienna", "56"),
        EmbThreadSew(123, 85, 74, "Sepia", "57"),
        EmbThreadSew(79, 57, 70, "Dark Sepia", "58"),
        EmbThreadSew(82, 58, 151, "Violet Blue", "59"),
        EmbThreadSew(0, 0, 160, "Blue Ink", "60"),
        EmbThreadSew(0, 150, 222, "Solar Blue", "61"),
        EmbThreadSew(178, 221, 83, "Green Dust", "62"),
        EmbThreadSew(250, 143, 187, "Crimson", "63"),
        EmbThreadSew(222, 100, 158, "Floral Pink", "64"),
        EmbThreadSew(181, 80, 102, "Wine", "65"),
        EmbThreadSew(94, 87, 71, "Olive Drab", "66"),
        EmbThreadSew(76, 136, 31, "Meadow", "67"),
        EmbThreadSew(228, 220, 121, "Canary Yellow", "68"),
        EmbThreadSew(203, 138, 26, "Toast", "69"),
        EmbThreadSew(198, 170, 66, "Beige", "70"),
        EmbThreadSew(236, 176, 44, "Honey Dew", "71"),
        EmbThreadSew(248, 128, 64, "Tangerine", "72"),
        EmbThreadSew(255, 229, 5, "Ocean Blue", "73"),
        EmbThreadSew(250, 122, 122, "Sepia", "74"),
        EmbThreadSew(107, 224, 0, "Royal Purple", "75"),
        EmbThreadSew(56, 108, 174, "Yellow Ocher", "76"),
        EmbThreadSew(208, 186, 176, "Beige Grey", "77"),
        EmbThreadSew(227, 190, 129, "Bamboo", "78"),
    ]


class EmbThreadSew(EmbThread):
    def __init__(self, red, green, blue, description, catalog_number):
        EmbThread.__init__(self)
        self.set_color(red, green, blue)
        self.description = description
        self.catalog_number = catalog_number
        self.brand = "Sew"
        self.chart = "Sew"
